-module(tree).
-compile([export_all,{no_auto_import,[max/2]}]).
-author('hanak@iit.bme.hu, modified by: kapolnai@iit.bme.hu').
-vsn('$LastChangedDate: 2020-10-29 09:58:17 +0100 (cs, 29 okt 2020) $$').

-type btree():: leaf | {any(), btree(), btree()}.

-spec empty() -> leaf.
% Üres fa.
empty() -> leaf.

-spec node(any(), btree(), btree()) -> btree().
% Lt és Rt fák összekapcsolása egy új V értékű csomóponttal.
node(V, Lt, Rt) -> {V,Lt,Rt}.

%max(X, Y) when X<Y -> Y;
%max(X, _Y)         -> X.

-spec depth(btree()) -> integer().
% Fa mélysége.
depth(leaf)      -> 0;
depth({_,Lt,Rt}) -> 1 + erlang:max(depth(Lt), depth(Rt)).

-spec nof_lvs(btree()) -> integer().
% Fa leveleinek száma (number of leaves).
nof_lvs(leaf)      -> 1;
nof_lvs({_,Lt,Rt}) -> nof_lvs(Lt) + nof_lvs(Rt).

%% ilist(N) -> lists:seq(1,N,1).
%% alist(F,N) -> lists:map(fun(X) -> list_to_atom([X-1+F]) end, ilist(N)).

-spec to_list_inord(btree()) -> list().
% Fa listává alakítása inord bejárással.
to_list_inord(leaf) ->
    [];
to_list_inord({V,Lt,Rt}) ->
    to_list_inord(Lt) ++ ( [V] ++ to_list_inord(Rt) ).

-spec to_list_preord(btree()) -> list().
% Fa listává alakítása preord bejárással.
to_list_preord(leaf) ->
    [];
to_list_preord({V,Lt,Rt}) ->
    [V] ++ to_list_preord(Lt) ++ to_list_preord(Rt).

-spec from_list_inord(list()) -> btree().
% Listából felépít egy fát inorder sorrendben.
from_list_inord([]) ->
    empty();
from_list_inord(L) ->
    {L1,[X|L2]} = lists:split(length(L) div 2, L),
    node(X, from_list_inord(L1), from_list_inord(L2)).

-spec from_list_preord(list()) -> btree().
% Listából felépít egy fát preorder sorrendben.
from_list_preord([]) ->
    empty();
from_list_preord([H|T]) ->
    {L1,L2} = lists:split(length(T) div 2, T),
    node(H, from_list_preord(L1), from_list_preord(L2)).

test(1) ->
%%                 1
%%             2       5
%%           3   4   6   7
%%          - - - - - - - -
    L = empty(),
    node(1,
         node(2,
              node(3,L,L),
              node(4,L,L)
             ),
         node(5,
              node(6,L,L),
              node(7,L,L)
             )
        );
test(2) ->
    from_list_inord([a,b,c,d,e,f,g]);
test(3) ->
    to_list_inord(test(2));
test(4) ->
    from_list_preord([a,b,c,d,e,f,g]);
test(5) ->
    to_list_preord(test(4)).
